//
//  IscPlayerView.m
//  iscflutterplugin
//
//  Created by 李晓康 on 2020/5/7.
//

#import "IscPlayerView.h"
#import "WFVideoHeader.h"
@import Photos;

@interface IscPlayerView ()

@property(nonatomic, strong)NSString * startTime;
@property(nonatomic, strong)NSString * endTime;

@property(nonatomic, strong)NSString * seekTime;

@property(nonatomic, strong)NSString * videoUrl;
@property(nonatomic, strong)HVPPlayer * player;

@property (nonatomic, assign) BOOL isPlaying;
@property (nonatomic, assign) BOOL isRecording;
@property (nonatomic, copy) NSString * recordPath;      // 录像保存路径

@end

@implementation IscPlayerView{
    
    // IscPlayerView 创建后的标识
    int64_t _viewId;
    NSNumber * _mPlayerStatus;
    // 消息回调
    FlutterMethodChannel* _channel;
    FlutterEventChannel * _eventChannel;
    FlutterEventSink _eventSink;
}

- (HVPPlayer *)player {
    if (!_player) {
        _player = [[HVPPlayer alloc] initWithPlayView:self];
        _player.delegate = self;
    }
    return _player;
}

- (instancetype)initWithWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId arguments:(id)args binaryMessenger:(NSObject<FlutterBinaryMessenger> *)messenger {
    if ([super init]) {
        
        NSString* channelName = [NSString stringWithFormat:@"plugin_isc_player_%lld", viewId];
        NSLog(@"+++++++%@", channelName);
        _channel = [FlutterMethodChannel methodChannelWithName:channelName binaryMessenger:messenger];
        __weak __typeof__(self) weakSelf = self;
        [_channel setMethodCallHandler:^(FlutterMethodCall *  call, FlutterResult  result) {
            [weakSelf onMethodCall:call result:result];
        }];
        
         NSString* eventChannelName = [NSString stringWithFormat:@"event_isc_player_%lld", viewId];
         _eventChannel = [FlutterEventChannel eventChannelWithName:eventChannelName  binaryMessenger:messenger];
         [_eventChannel setStreamHandler:self];
        
        
        [self registNotifications];
        self.isRecording = NO;
        self.isPlaying = NO;
        
        _mPlayerStatus = [NSNumber numberWithInt:IDLE];
        
    }
    return self;
}

- (void)registNotifications {
    // 注册前后台切换通知
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillResignActive) name:UIApplicationWillResignActiveNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationDidBecomeActive) name:UIApplicationDidBecomeActiveNotification object:nil];
}

-(void)onMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result{
    
//    if (self.isRecording) {
//
//        [self recordVideo:NO];
//    }
//    // 退出当前页面，需要停止播放
//    if (self.isPlaying) {
//        NSLog(@"停止播放当前视频");
//        [self startPlayVideo:NO];
//
//    }
    
//    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
//    });
    // 开始预览
    NSLog(@"========%@", call);
    if ([[call method] isEqualToString:@"startRealPlay"]) {
        //获取参数
        NSDictionary *dict = call.arguments;
        self.videoUrl = dict[@"url"];
        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"changeStream"]) {        // 码流平滑切换
        //获取参数
        NSDictionary *dict = call.arguments;
        self.videoUrl = dict[@"url"];
//        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"startPlayback"]) {       // 开始回放
        //获取参数
        NSDictionary *dict = call.arguments;
        self.videoUrl = dict[@"url"];
        self.startTime = dict[@"startTime"];
        self.endTime = dict[@"stopTime"];
//        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"seekAbsPlayback"]) {         // 按绝对时间回放定位
        //获取参数
        NSDictionary *dict = call.arguments;
        self.seekTime = dict[@"seekTime"];
//        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"getOSDTime"]) {      // 查询当前播放时间戳接口
        //获取参数
        NSDictionary *dict = call.arguments;
//        self.videoUrl = dict[@"url"];
//        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"pause"]) {       // 暂停回放
        //获取参数
        NSDictionary *dict = call.arguments;
//        self.videoUrl = dict[@"url"];
//        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"resume"]) {          // 恢复回放
        //获取参数
        NSDictionary *dict = call.arguments;
//        self.videoUrl = dict[@"url"];
//        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"stopPlay"]) {          // 停止播放
        //获取参数
        NSDictionary *dict = call.arguments;
        self.videoUrl = dict[@"url"];
        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"startVoiceTalk"]) {          // 开启语音对讲
        //获取参数
        NSDictionary *dict = call.arguments;
        self.videoUrl = dict[@"url"];
//        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"stopVoiceTalk"]) {          // 关闭语音对讲
        //获取参数
        NSDictionary *dict = call.arguments;
        self.videoUrl = dict[@"url"];
        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"capturePicture"]) {          // 抓图
        //获取参数
        NSDictionary *dict = call.arguments;
        self.videoUrl = dict[@"url"];
        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"startRecord"]) {          // 开启本地录像
        //获取参数
        NSDictionary *dict = call.arguments;
        self.videoUrl = dict[@"url"];
        [self record:YES];
        
    } if ([[call method] isEqualToString:@"stopRecord"]) {          // 关闭本地录像
        //获取参数
        NSDictionary *dict = call.arguments;
        self.videoUrl = dict[@"url"];
        [self record:NO];
        
    } if ([[call method] isEqualToString:@"enableSound"]) {          // 声音控制
        //获取参数
        NSDictionary *dict = call.arguments;
        self.videoUrl = dict[@"url"];
        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"onResume"]) {          //
        //获取参数
        NSDictionary *dict = call.arguments;
        self.videoUrl = dict[@"url"];
        [self startPlayVideo:YES];
        
    } if ([[call method] isEqualToString:@"onPause"]) {          //
        
        
    } else {
        //其他方法的回调
        result(FlutterMethodNotImplemented);
    }
}

- (nonnull UIView *)view {
    return self;
}

- (NSObject<FlutterMessageCodec>*)createArgsCodec {
    return [FlutterStandardMessageCodec sharedInstance];
}

#pragma mark - 视频相关操作
/// 开始播放视频
/// @param startType yes开始，no结束
- (void)startPlayVideo:(BOOL)startType {
    if (startType) {
        
        if (self.videoUrl == 0) {
            NSString * msg = @"预览URL为空";
        }
        
        // 为避免卡顿，开启预览可以放到子线程中，在应用中灵活处理
        dispatch_sync(dispatch_get_global_queue(0, 0), ^{
            if ([self.player startRealPlay:self.videoUrl]) {
                // TODO: 开始播放
                self.isPlaying = YES;
            }
        });
        return;
    } else {
        if (_isRecording) {
            //如果在录像，先关闭录像
            [self recordVideo:NO];
        }
        dispatch_sync(dispatch_get_global_queue(0, 0), ^{
            NSError * error;
            [self.player stopPlay:&error];
            NSLog(@"%@", error);
            if (!error) {
                _isPlaying = NO;
            }
        });
        
    }
    
}
// 抓图
- (void)capturePicture {
    if (!_isPlaying) {
        NSString * msg = @"未播放视频，不能抓图";
        return;
    }
    [PHPhotoLibrary requestAuthorization:^(PHAuthorizationStatus status) {
        dispatch_async(dispatch_get_main_queue(), ^{
            if (status == PHAuthorizationStatusDenied) {
                NSString * msg = @"无保存图片到相册的权限，不能抓图";
            }
            else {
                [self capture];
            }
        });
    }];
}

- (void)capture {
    if (!_isPlaying) {
        return;
    }
    // 生成图片路径
    NSString *documentDirectorie = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES).firstObject;
    NSString *filePath = [documentDirectorie stringByAppendingFormat:@"/%.f.jpg", [NSDate date].timeIntervalSince1970];
    NSError *error;
    if (![_player capturePicture:filePath error:&error]) {
        NSString *message = [NSString stringWithFormat:@"抓图失败，错误码是 0x%08lx", error.code];
        
    }
    else {
        [[PHPhotoLibrary sharedPhotoLibrary] performChanges:^{
            [PHAssetChangeRequest creationRequestForAssetFromImageAtFileURL:[NSURL URLWithString:filePath]];
        } completionHandler:^(BOOL success, NSError * _Nullable error) {
            NSString *message;
            if (success) {
                message = @"抓图成功，并保存到系统相册";
            }
            else {
                message = @"保存到系统相册失败";
            }
            [[NSFileManager defaultManager] removeItemAtPath:filePath error:nil];
            dispatch_async(dispatch_get_main_queue(), ^{
                // 提示错误消息
            });
        }];
    }
}

// 录像
- (void)record:(BOOL)sender {
    if (!_isPlaying) {
        NSString * msg = @"未播放视频，不能录像";
        return;
    }
    [PHPhotoLibrary requestAuthorization:^(PHAuthorizationStatus status) {
        if (status == PHAuthorizationStatusDenied) {
            dispatch_async(dispatch_get_main_queue(), ^{
                NSString * msg = @"无保存录像到相册的权限，不能录像";
            });
        }
        else {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self recordVideo:sender];
            });
        }
    }];
}

/// 开始录像
/// @param recordType 类型，yes开始录像，no结束录像
- (void)recordVideo:(BOOL)recordType {
    if (!_isPlaying) {
        return;
    }
    NSError *error;
    // 开始录像
    if (recordType) {
        // 生成图片路径
        NSString *documentDirectorie = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES).firstObject;
        NSString *filePath = [documentDirectorie stringByAppendingFormat:@"/%.f.mp4", [NSDate date].timeIntervalSince1970];
        _recordPath = [filePath copy];
        if ([_player startRecord:filePath error:&error]) {
            _isRecording = YES;
            // TODO:开始录像，改变按钮状态
            
        } else {
            NSString *message = [NSString stringWithFormat:@"开始录像失败，错误码是 0x%08lx", error.code];
            dispatch_async(dispatch_get_main_queue(), ^{
                // TODO:提示错误消息
            });
        }
        return;
    }
    if (!_isRecording) {
        return;
    }
    // 停止录像
    if ([_player stopRecord:&error]) {
        _isRecording = NO;
        // TODO:停止录像，改变按钮状态
        //可在自定义recordPath路径下取录像文件
        [[PHPhotoLibrary sharedPhotoLibrary] performChanges:^{
            [PHAssetChangeRequest creationRequestForAssetFromVideoAtFileURL:[NSURL URLWithString:_recordPath]];
        } completionHandler:^(BOOL success, NSError * _Nullable error) {
            NSString *message;
            if (success) {
                message = @"录像成功，并保存到系统相册";
                
            }
            else {
                message = @"保存到系统相册失败";
            }
            dispatch_async(dispatch_get_main_queue(), ^{
//                [MBProgressHUD showTipMessageInView:message];
            });
        }];
    }
    else {
        NSString *message = [NSString stringWithFormat:@"停止录像失败，错误码是 0x%08lx", error.code];
        dispatch_async(dispatch_get_main_queue(), ^{
//            [MBProgressHUD showTipMessageInView:message];
        });
    }
}

#pragma mark - HVPPlayerDelegate
- (void)player:(HVPPlayer *)player playStatus:(HVPPlayStatus)playStatus errorCode:(HVPErrorCode)errorCode {
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        self.isPlaying = NO;
        NSString *message;
        NSLog(@"%ld---%ld", playStatus, errorCode);
        // 预览时，没有HVPPlayStatusFinish状态，该状态表明录像片段已播放完
        switch (playStatus) {
            case HVPPlayStatusSuccess: {
                self.isPlaying = YES;
                _mPlayerStatus = [NSNumber numberWithInt:SUCCESS];
                // 默认开启声音
                [self.player enableSound:YES error:nil];
                dispatch_sync(dispatch_get_main_queue(), ^{
                    
                });
            }
                break;
            case HVPPlayStatusFailure: {
                _mPlayerStatus = [NSNumber numberWithInt:FAILED];
                if (errorCode == HVPErrorCodeURLInvalid) {
                    message = @"URL输入错误请检查URL或者URL已失效请更换URL";
                } else {
                    message = [NSString stringWithFormat:@"开启预览失败, 错误码是 : 0x%08lx", errorCode];
                }
            }
                break;
            case HVPPlayStatusException: {
                // 预览过程中出现异常, 可能是取流中断，可能是其他原因导致的，具体根据错误码进行区分
                // 做一些提示操作
                _mPlayerStatus = [NSNumber numberWithInt:EXCEPTION];
                message = [NSString stringWithFormat:@"播放异常, 错误码是 : 0x%08lx", errorCode];
                if (self.isRecording) {
                    //如果在录像，先关闭录像
                    [self recordVideo:NO];
                }
                // 关闭播放
                [self.player stopPlay:nil];
            }
                break;
                
            default:
                break;
        }
        NSDictionary * ret = @{@"status" : _mPlayerStatus};
        NSLog(@"%@", ret);
        [_channel invokeMethod:@"onPlayerStatusCallback" arguments:ret];
        if (message) {
            dispatch_async(dispatch_get_main_queue(), ^{
//                [MBProgressHUD showTipMessageInView:message];
            });
        }
    });
}

#pragma mark - Notification Method
- (void)applicationWillResignActive {
    if (_isRecording) {
        [self recordVideo:NO];
    }
    if (_isPlaying) {
        [self startPlayVideo:NO];
    }
}

- (void)applicationDidBecomeActive {
    NSLog(@"切换为前台");
    if (self.videoUrl.length > 0) {
        NSLog(@"%@", self.videoUrl);
        // 为避免卡顿，开启预览可以放到子线程中，在应用中灵活处理
        dispatch_sync(dispatch_get_global_queue(0, 0), ^{
            if ([self.player startRealPlay:self.videoUrl]) {
                NSLog(@"开始播放");
                self.isPlaying = YES;
            }
        });
    }
}

@end
