import 'dart:async';

import 'package:flutter/services.dart';

///闲置状态
const IDLE = 0;

///加载中状态
const int LOADING = 1;

///播放成功
const int SUCCESS = 2;

///暂时停止播放
const int STOPPING = 3;

///播放失败
const int FAILED = 4;

///播放过程中出现异常
const int EXCEPTION = 5;

///回放结束
const int FINISH = 6;

///播放器创建成功的回调
typedef void IscPlayerCreatedCallback(Iscflutterplugin controller);

///播放器状态监听回调
typedef void IscPlayerStatusCallback(int status);

///播放器控制器
class Iscflutterplugin {
//  static const MethodChannel _channel =
//      const MethodChannel('iscflutterplugin');
//
//  static Future<String> get platformVersion async {
//    final String version = await _channel.invokeMethod('getPlatformVersion');
//    return version;
//  }

  MethodChannel _channel;

  Iscflutterplugin.init(int id) {
    _channel = MethodChannel('plugin_isc_player_$id');
  }

  ///开始预览
  Future<void> startRealPlay(String url) async {
    return _channel.invokeMethod('startRealPlay', <String, dynamic>{
      "url": url,
    });
  }

  ///码流平滑切换
  Future<void> changeStream(String url) async {
    return _channel.invokeMethod('changeStream', <String, dynamic>{
      "url": url,
    });
  }

  ///开始回放
  Future<void> startPlayback(
      String url, String startTime, String stopTime) async {
    return _channel.invokeMethod('startPlayback', <String, dynamic>{
      "url": url,
      "startTime": startTime,
      "stopTime": stopTime,
    });
  }

  ///按绝对时间回放定位
  Future<void> seekAbsPlayback(String seekTime) async {
    return _channel.invokeMethod('seekAbsPlayback', <String, dynamic>{
      "seekTime": seekTime,
    });
  }

  ///查询当前播放时间戳接口
  Future<String> getOSDTime() async {
    return _channel.invokeMethod('getOSDTime');
  }

  ///暂停回放
  Future<bool> pause() async {
    return _channel.invokeMethod('pause');
  }

  ///恢复回放
  Future<bool> resume() async {
    return _channel.invokeMethod('resume');
  }

  ///停止播放
  Future<bool> stopPlay() async {
    return _channel.invokeMethod('stopPlay');
  }

  ///开启语音对讲
  Future<void> startVoiceTalk(String url) async {
    return _channel.invokeMethod('startVoiceTalk', <String, dynamic>{
      "url": url,
    });
  }

  ///关闭语音对讲
  Future<bool> stopVoiceTalk() async {
    return _channel.invokeMethod('stopVoiceTalk');
  }

  ///预览/回放 抓图
  Future<bool> capturePicture(String bitmapPath) async {
    return _channel.invokeMethod('capturePicture', <String, dynamic>{
      "path": bitmapPath,
    });
  }

  ///开启本地录像
  Future<bool> startRecord(String mediaFilePath) async {
    return _channel.invokeMethod('startRecord', <String, dynamic>{
      "path": mediaFilePath,
    });
  }

  ///关闭本地录像
  Future<bool> stopRecord() async {
    return _channel.invokeMethod('stopRecord');
  }

  ///声音控制
  Future<bool> enableSound(bool enable) async {
    return _channel.invokeMethod('enableSound', <String, dynamic>{
      "enable": enable,
    });
  }

  ///生命周期回调
  ///注意:APP前后台切换时 华为手机 上不会回调生命周期方法，例如：华为P20，可以在这里手动调用
  Future<void> onResume() async {
    return _channel.invokeMethod('onResume');
  }

  ///生命周期回调
  ///注意:APP前后台切换时 华为手机 上不会回调生命周期方法，例如：华为P20，可以在这里手动调用
  Future<void> onPause() async {
    return _channel.invokeMethod('onPause');
  }

  ///设置播放器回调监听
  Future<dynamic> setStatusCallback(IscPlayerStatusCallback callback) async {
    _channel.setMethodCallHandler((methodCall) async {
      Map arg = methodCall.arguments;
      print('接受到了Native消息:$arg');

      switch (methodCall.method) {
        case "onPlayerStatusCallback":
        case "onTalkStatusCallback":
          int status = arg['status'];
          callback(status);
          break;
        default:
      }
    });
  }

  ///获取错误码信息
  String getStatusMessage(int status) {
    String msg;
    switch (status) {
      case IDLE:
        msg = "闲置状态";
        break;
      case LOADING:
        msg = "加载中状态";
        break;
      case SUCCESS:
        msg = "播放成功";
        break;
      case STOPPING:
        msg = "暂时停止播放";
        break;
      case FAILED:
        msg = "播放失败";
        break;
      case EXCEPTION:
        msg = "播放过程中出现异常";
        break;
      case FINISH:
        msg = "回放结束";
        break;
    }
    return msg;
  }
}
